"""Helpers for using Playwright from notebooks and more"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../00_core.ipynb.

# %% auto 0
__all__ = ['get_page', 'page_ready', 'frames_ready', 'wait_page', 'get_full_content', 'read_page_async', 'read_page', 'h2md',
           'url2md_async', 'url2md', 'get2md']

# %% ../00_core.ipynb
from fastcore.utils import *
from fastcore.meta import delegates
import uuid, warnings

from playwright.async_api import async_playwright, TimeoutError as PTimeoutError
from playwright_stealth import stealth_async
from anyio import from_thread
from httpx import get

from bs4 import BeautifulSoup, GuessedAtParserWarning
from html2text import HTML2Text

# %% ../00_core.ipynb
async def get_page(*args, **kwargs):
    p = await async_playwright().start()
    c = await p.chromium.launch(*args, **kwargs)
    ctx = await c.new_context()
    page = await ctx.new_page()
    page.stop = p.stop
    await stealth_async(page)
    return page

# %% ../00_core.ipynb
async def page_ready(page, pause=50, timeout=5000):
    "Waith until main content of `page` is ready"
    await page.wait_for_load_state('domcontentloaded')
    await page.wait_for_load_state('networkidle')
    await page.wait_for_timeout(pause)
    try: await page.wait_for_selector('meta', state="attached", timeout=timeout)
    except PTimeoutError as e: pass
    await page.wait_for_timeout(pause)

# %% ../00_core.ipynb
async def frames_ready(page, pause=50, timeout=5000):
    "Wait until all visible frames (if any) on `page` are ready"
    iframes = await page.query_selector_all('iframe:visible')
    if not iframes: return
    for iframe in iframes:
        await iframe.wait_for_element_state('visible', timeout=timeout)
        await page.wait_for_timeout(pause)
        frame = await iframe.content_frame()
        if frame:
            await frame.wait_for_load_state('domcontentloaded', timeout=timeout)
            await frame.wait_for_load_state('networkidle', timeout=timeout)

# %% ../00_core.ipynb
async def wait_page(page, pause=50, timeout=5000):
    "Wait until page and visible frames (if any) on `page` are ready"
    await page_ready(page, pause=pause, timeout=timeout)
    await frames_ready(page, pause=pause, timeout=timeout)

# %% ../00_core.ipynb
async def get_full_content(page):
    "Tuple of page content and dict of frames' content"
    main_content = await page.content()
    iframes = await page.query_selector_all('iframe')
    iframe_contents = {}
    for iframe in iframes:
        frame = await iframe.content_frame()
        if frame:
            key = await iframe.get_attribute('id') or str(uuid.uuid4())
            iframe_contents[key] = await frame.content()
    return main_content, iframe_contents

# %% ../00_core.ipynb
async def read_page_async(url, pause=50, timeout=5000, page=None):
    "Return contents of `url` and its iframes using Playwright async"
    has_page = bool(page)
    if not page: page = await get_page()
    try:
        await page.goto(url)
        await wait_page(page, pause=pause, timeout=timeout)
        return await get_full_content(page)
    finally:
        if not has_page: await page.close()

# %% ../00_core.ipynb
def read_page(url, pause=50, timeout=5000, page=None):
    "Return contents of `url` and its iframes using Playwright"
    with from_thread.start_blocking_portal() as p: return p.call(read_page_async, url, pause, timeout, page)

# %% ../00_core.ipynb
def h2md(h):
    "Convert HTML `h` to markdown using `HTML2Text"
    h2t = HTML2Text(bodywidth=5000)
    h2t.ignore_links = True
    h2t.mark_code = True
    h2t.ignore_images = True
    return h2t.handle(str(h))

# %% ../00_core.ipynb
async def url2md_async(url, sel=None, pause=50, timeout=5000, page=None):
    "Read `url` with `read_page`, optionally selecting CSS selector `sel`"
    warnings.filterwarnings("ignore", category=GuessedAtParserWarning)
    cts,_ = await read_page_async(url, pause, timeout=timeout, page=page)
    soup = BeautifulSoup(cts)
    content = soup.select_one(sel)
    return h2md(content)

# %% ../00_core.ipynb
def url2md(url, sel=None, pause=50, timeout=5000, page=None):
    "Read `url` with `read_page`"
    warnings.filterwarnings("ignore", category=GuessedAtParserWarning)
    cts,_ = read_page(url, pause, timeout=timeout, page=page)
    soup = BeautifulSoup(cts)
    content = soup.select_one(sel)
    return h2md(content)

# %% ../00_core.ipynb
@delegates(get)
def get2md(url, sel=None, **kwargs):
    "Read `url` with `httpx.get`"
    warnings.filterwarnings("ignore", category=GuessedAtParserWarning)
    cts = get(url, **kwargs)
    soup = BeautifulSoup(cts)
    content = soup.select_one(sel)
    return h2md(content)
